#include <ESP32-HUB75-MatrixPanel-I2S-DMA.h>

#define PANEL_RES_X 64 // 屏幕宽度
#define PANEL_RES_Y 64 // 屏幕高度
#define PANEL_CHAIN 2  // 两块屏幕拼接

MatrixPanel_I2S_DMA *dma_display = nullptr;

// 贪食蛇效果参数
#define SNAKE_LENGTH 64 // 蛇的长度
#define SNAKE_WIDTH 3 // 蛇的身体宽度，单位：像素
#define SNAKE_HEAD_SIZE 3 // 蛇头的大小：3x3 像素

// 定义 5 条蛇
uint8_t snake_buffers[5][SNAKE_LENGTH][2]; // 5 条蛇的坐标
int directionsX[5] = {1, 1, 1, 1, 1}; // 每条蛇的 X 方向
int directionsY[5] = {0, 0, 0, 0, 0}; // 每条蛇的 Y 方向

// 线性插值函数
uint8_t lerp(uint8_t start, uint8_t end, float t) {
  return (uint8_t)(start + (end - start) * t);
}

// 生成渐变颜色函数
void getRainbowColor(int index, uint8_t &r, uint8_t &g, uint8_t &b, int snakeIndex) {
  // 定义蛇头和蛇尾的颜色
  uint8_t r_start = 255, g_start = 100, b_start = 0;  // 头部暖色 (红色到黄色)
  uint8_t r_end = 0, g_end = 0, b_end = 255;  // 尾部冷色 (蓝色)
  
  // 为每条蛇设置不同的颜色
  switch (snakeIndex) {
    case 0:
      r_start = 255; g_start = 0; b_start = 0; // 红色
      r_end = 255; g_end = 165; b_end = 0; // 橙色
      break;
    case 1:
      r_start = 255; g_start = 255; b_start = 0; // 黄色
      r_end = 0; g_end = 255; b_end = 0; // 绿色
      break;
    case 2:
      r_start = 0; g_start = 255; b_start = 0; // 绿色
      r_end = 0; g_end = 0; b_end = 255; // 蓝色
      break;
    case 3:
      r_start = 0; g_start = 0; b_start = 255; // 蓝色
      r_end = 255; g_end = 0; b_end = 255; // 紫色
      break;
    case 4:
      r_start = 255; g_start = 0; b_start = 255; // 紫色
      r_end = 255; g_end = 255; b_end = 255; // 白色
      break;
  }

  // 计算渐变因子
  float t = (float)index / (SNAKE_LENGTH - 1);  // 从头部到尾部逐渐变化
  
  // 线性插值计算颜色
  r = lerp(r_start, r_end, t);
  g = lerp(g_start, g_end, t);
  b = lerp(b_start, b_end, t);
}

// 清除蛇的路径
void clearSnake(uint8_t snake_buffer[SNAKE_LENGTH][2]) {
  for (int i = 0; i < SNAKE_LENGTH; i++) {
    int x = snake_buffer[i][0];
    int y = snake_buffer[i][1];
    // 清除每段蛇的位置
    for (int dx = 0; dx < SNAKE_WIDTH; dx++) {
      for (int dy = 0; dy < SNAKE_WIDTH; dy++) {
        dma_display->drawPixelRGB888(x + dx, y + dy, 0, 0, 0); // 清空像素
      }
    }
  }
}

// 绘制蛇，头部为白色，其余部分为渐变颜色
void drawSnake(uint8_t snake_buffer[SNAKE_LENGTH][2], int snakeIndex) {
  for (int i = 0; i < SNAKE_LENGTH; i++) {
    int x = snake_buffer[i][0];
    int y = snake_buffer[i][1];

    uint8_t r, g, b;
    // 计算每一段蛇的颜色，从头到尾渐变
    getRainbowColor(i, r, g, b, snakeIndex);

    // 为蛇的头部绘制固定白色，宽度为 SNAKE_HEAD_SIZE
    if (i == 0) {
      // 绘制蛇头，确保它是白色，蛇头是 3x3 大小
      for (int dx = 0; dx < SNAKE_HEAD_SIZE; dx++) {
        for (int dy = 0; dy < SNAKE_HEAD_SIZE; dy++) {
          dma_display->drawPixelRGB888(x + dx, y + dy, 255, 255, 255); // 白色
        }
      }
    } else {
      // 绘制蛇身体部分
      for (int dx = 0; dx < SNAKE_WIDTH; dx++) {
        for (int dy = 0; dy < SNAKE_WIDTH; dy++) {
          dma_display->drawPixelRGB888(x + dx, y + dy, r, g, b); // 使用渐变颜色
        }
      }
    }
  }
}

void setup() {
  // 配置 LED 矩阵的引脚
  HUB75_I2S_CFG::i2s_pins _pins = {
    25, 26, 27, 14, 12, 13, 23, 19, 5, 17, 18, 4, 15, 16,
  };

  // 初始化屏幕配置
  HUB75_I2S_CFG mxconfig(PANEL_RES_X, PANEL_RES_Y, PANEL_CHAIN, _pins);

  // 初始化 DMA 显示对象
  dma_display = new MatrixPanel_I2S_DMA(mxconfig);
  dma_display->begin();
  dma_display->clearScreen();
  dma_display->setBrightness(50); // 设置亮度 (0-255)

  // 初始化 5 条蛇的起始位置
  for (int i = 0; i < 5; i++) {
    for (int j = 0; j < SNAKE_LENGTH; j++) {
      snake_buffers[i][j][0] = 30 + j; // 初始 x 坐标
      snake_buffers[i][j][1] = 30 + i * 10; // 每条蛇起始 y 坐标有偏移
    }
  }
}

void loop() {
  // 清除每条蛇之前的路径
  for (int i = 0; i < 5; i++) {
    clearSnake(snake_buffers[i]);
  }

  // 更新每条蛇的位置
  for (int i = 0; i < 5; i++) {
    for (int j = SNAKE_LENGTH - 1; j > 0; j--) {
      snake_buffers[i][j][0] = snake_buffers[i][j - 1][0];
      snake_buffers[i][j][1] = snake_buffers[i][j - 1][1];
    }
    snake_buffers[i][0][0] += directionsX[i];
    snake_buffers[i][0][1] += directionsY[i];
  }

  // 随机改变每条蛇的移动方向
  for (int i = 0; i < 5; i++) {
    if (random(0, 100) < 10) {
      int newDirectionX = random(-1, 2);
      int newDirectionY = random(-1, 2);
      if (newDirectionX == 0 && newDirectionY == 0) {
        newDirectionX = directionsX[i];
        newDirectionY = directionsY[i];
      }
      if (newDirectionX != 0 && newDirectionY != 0) {
        newDirectionY = 0;
      }
      if ((newDirectionX == -directionsX[i] && directionsX[i] != 0) || (newDirectionY == -directionsY[i] && directionsY[i] != 0)) {
        newDirectionX = directionsX[i];
        newDirectionY = directionsY[i];
      }
      directionsX[i] = newDirectionX;
      directionsY[i] = newDirectionY;
    }
  }

  // 检查每条蛇是否碰到屏幕边界
  for (int i = 0; i < 5; i++) {
    if (snake_buffers[i][0][0] < 0) {
      snake_buffers[i][0][0] = PANEL_RES_X * PANEL_CHAIN - 1;
    }
    if (snake_buffers[i][0][0] >= PANEL_RES_X * PANEL_CHAIN) {
      snake_buffers[i][0][0] = 0;
    }
    if (snake_buffers[i][0][1] < 0) {
      snake_buffers[i][0][1] = PANEL_RES_Y - 1;
    }
    if (snake_buffers[i][0][1] >= PANEL_RES_Y) {
      snake_buffers[i][0][1] = 0;
    }
  }

  // 绘制 5 条蛇
  for (int i = 0; i < 5; i++) {
    drawSnake(snake_buffers[i], i);
  }

  // 延迟 100ms
  delay(10);
}